@@ -11,6 +11,7 @@
 __docformat__ = 'restructuredtext'
 
 
+from copy import copy
 import logging
 from functools import partial
 from itertools import dropwhile
@@ -24,6 +25,7 @@ from datalad.interface.base import Interface
 from datalad.interface.utils import eval_results
 from datalad.interface.base import build_doc
 from datalad.interface.results import get_status_dict
+from datalad.core.local.run import assume_ready_opt
 from datalad.core.local.run import run_command
 from datalad.core.local.run import format_command
 from datalad.core.local.run import _format_cmd_shorty
@@ -33,6 +35,7 @@ from datalad.consts import PRE_INIT_COMMIT_SHA
 from datalad.support.constraints import EnsureNone, EnsureStr
 from datalad.support.param import Parameter
 from datalad.support.json_py import load_stream
+from datalad.support.exceptions import CapturedException
 
 from datalad.distribution.dataset import require_dataset
 from datalad.distribution.dataset import EnsureDataset
@@ -40,6 +43,11 @@ from datalad.distribution.dataset import datasetmethod
 
 lgr = logging.getLogger('datalad.interface.rerun')
 
+rerun_assume_ready_opt = copy(assume_ready_opt)
+rerun_assume_ready_opt._doc += """
+Note that this option also affects any additional outputs that are
+automatically inferred based on inspecting changed files in the run commit."""
+
 
 @build_doc
 class Rerun(Interface):
@@ -85,9 +93,11 @@ class Rerun(Interface):
             nargs="?",
             doc="""rerun command(s) in `revision`. By default, the command from
             this commit will be executed, but [CMD: --since CMD][PY: `since`
-            PY] can be used to construct a revision range.""",
-            default="HEAD",
-            constraints=EnsureStr()),
+            PY] can be used to construct a revision range. The default value is
+            like "HEAD" but resolves to the main branch when on an adjusted
+            branch.""",
+            default=None,
+            constraints=EnsureStr() | EnsureNone()),
         since=Parameter(
             args=("--since",),
             doc="""If `since` is a commit-ish, the commands from all commits
@@ -142,6 +152,7 @@ class Rerun(Interface):
             doc="""Don't actually re-execute anything, just display what would
             be done. [CMD: Note: If you give this option, you most likely want
             to set --output-format to 'json' or 'json_pp'. CMD]"""),
+        assume_ready=rerun_assume_ready_opt,
         explicit=Parameter(
             args=("--explicit",),
             action="store_true",
@@ -179,7 +190,7 @@ class Rerun(Interface):
     @datasetmethod(name='rerun')
     @eval_results
     def __call__(
-            revision="HEAD",
+            revision=None,
             since=None,
             dataset=None,
             branch=None,
@@ -187,15 +198,17 @@ class Rerun(Interface):
             onto=None,
             script=None,
             report=False,
+            assume_ready=None,
             explicit=False):
 
         ds = require_dataset(
             dataset, check_installed=True,
-            purpose='rerunning a command')
+            purpose='rerun a command')
+        ds_repo = ds.repo
 
         lgr.debug('rerunning command output underneath %s', ds)
 
-        if script is None and not (report or explicit) and ds.repo.dirty:
+        if script is None and not (report or explicit) and ds_repo.dirty:
             yield get_status_dict(
                 'run',
                 ds=ds,
@@ -205,20 +218,34 @@ class Rerun(Interface):
                     'use `datalad status` to inspect unsaved changes'))
             return
 
-        if not ds.repo.get_hexsha():
+        if not ds_repo.get_hexsha():
             yield get_status_dict(
                 'run', ds=ds,
                 status='impossible',
                 message='cannot rerun command, nothing recorded')
             return
 
-        if branch and branch in ds.repo.get_branches():
+        # ATTN: Use get_corresponding_branch() rather than is_managed_branch()
+        # for compatibility with a plain GitRepo.
+        if (onto is not None or branch is not None) and \
+           ds_repo.get_corresponding_branch():
+            yield get_status_dict(
+                "run", ds=ds, status="impossible",
+                message=("--%s is incompatible with adjusted branch",
+                         "branch" if onto is None else "onto"))
+            return
+
+        if branch and branch in ds_repo.get_branches():
             yield get_status_dict(
                 "run", ds=ds, status="error",
                 message="branch '{}' already exists".format(branch))
             return
 
-        if not ds.repo.commit_exists(revision + "^"):
+        if revision is None:
+            revision = ds_repo.get_corresponding_branch() or \
+                ds_repo.get_active_branch() or "HEAD"
+
+        if not ds_repo.commit_exists(revision + "^"):
             # Only a single commit is reachable from `revision`.  In
             # this case, --since has no effect on the range construction.
             revrange = revision
@@ -235,14 +262,16 @@ class Rerun(Interface):
         elif report:
             handler = _report
         else:
-            handler = partial(_rerun, explicit=explicit)
+            handler = partial(_rerun, assume_ready=assume_ready,
+                              explicit=explicit)
 
         for res in handler(ds, results):
             yield res
 
 
 def _revrange_as_results(dset, revrange):
-    rev_lines = dset.repo.get_revisions(
+    ds_repo = dset.repo
+    rev_lines = ds_repo.get_revisions(
         revrange, fmt="%H %P", options=["--reverse", "--topo-order"])
     if not rev_lines:
         return
@@ -254,7 +283,7 @@ def _revrange_as_results(dset, revrange):
         fields = rev_line.strip().split(" ")
         rev, parents = fields[0], fields[1:]
         res = get_status_dict("run", ds=dset, commit=rev, parents=parents)
-        full_msg = dset.repo.format_commit("%B", rev)
+        full_msg = ds_repo.format_commit("%B", rev)
         try:
             msg, info = get_run_info(dset, full_msg)
         except ValueError as exc:
@@ -285,9 +314,12 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
     try:
         results = _revrange_as_results(dset, revrange)
     except ValueError as exc:
-        yield get_status_dict("run", status="error", message=exc_str(exc))
+        ce = CapturedException(exc)
+        yield get_status_dict("run", status="error", message=str(ce),
+                              exception=ce)
         return
 
+    ds_repo = dset.repo
     # Drop any leading commits that don't have a run command. These would be
     # skipped anyways.
     results = list(dropwhile(lambda r: "run_info" not in r, results))
@@ -300,7 +332,7 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
     if onto is not None and onto.strip() == "":
         onto = results[0]["commit"] + "^"
 
-    if onto and not dset.repo.commit_exists(onto):
+    if onto and not ds_repo.commit_exists(onto):
         yield get_status_dict(
             "run", ds=dset, status="error",
             message=("Revision specified for --onto (%s) does not exist.",
@@ -314,14 +346,14 @@ def _rerun_as_results(dset, revrange, since, branch, onto, message):
             ds=dset,
             # Resolve this to the full hexsha so downstream code gets a
             # predictable form.
-            commit=dset.repo.get_hexsha(start_point),
+            commit=ds_repo.get_hexsha(start_point),
             branch=branch,
             rerun_action="checkout",
             status="ok")
 
     def skip_or_pick(hexsha, result, msg):
         result["rerun_action"] = "skip-or-pick"
-        shortrev = dset.repo.get_hexsha(hexsha, short=True)
+        shortrev = ds_repo.get_hexsha(hexsha, short=True)
         result["message"] = (
             "%s %s; %s",
             shortrev, msg, "skipping or cherry picking")
@@ -353,12 +385,13 @@ def _mark_nonrun_result(result, which):
     return result
 
 
-def _rerun(dset, results, explicit=False):
+def _rerun(dset, results, assume_ready=None, explicit=False):
+    ds_repo = dset.repo
     # Keep a map from an original hexsha to a new hexsha created by the rerun
     # (i.e. a reran, cherry-picked, or merged commit).
     new_bases = {}  # original hexsha => reran hexsha
-    branch_to_restore = dset.repo.get_active_branch()
-    head = onto = dset.repo.get_hexsha()
+    branch_to_restore = ds_repo.get_active_branch()
+    head = onto = ds_repo.get_hexsha()
     for res in results:
         lgr.info(_get_rerun_log_msg(res))
         rerun_action = res.get("rerun_action")
@@ -375,8 +408,8 @@ def _rerun(dset, results, explicit=False):
             else:
                 checkout_options = ["--detach"]
                 branch_to_restore = None
-            dset.repo.checkout(res_hexsha,
-                               options=checkout_options)
+            ds_repo.checkout(res_hexsha,
+                             options=checkout_options)
             head = onto = res_hexsha
             continue
 
@@ -392,23 +425,23 @@ def _rerun(dset, results, explicit=False):
             old_parents = res["parents"]
             new_parents = [new_bases.get(p, p) for p in old_parents]
             if old_parents == new_parents:
-                if not dset.repo.is_ancestor(res_hexsha, head):
-                    dset.repo.checkout(res_hexsha)
+                if not ds_repo.is_ancestor(res_hexsha, head):
+                    ds_repo.checkout(res_hexsha)
             elif res_hexsha != head:
-                if dset.repo.is_ancestor(res_hexsha, onto):
+                if ds_repo.is_ancestor(res_hexsha, onto):
                     new_parents = [p for p in new_parents
-                                   if not dset.repo.is_ancestor(p, onto)]
+                                   if not ds_repo.is_ancestor(p, onto)]
                 if new_parents:
                     if new_parents[0] != head:
                         # Keep the direction of the original merge.
-                        dset.repo.checkout(new_parents[0])
+                        ds_repo.checkout(new_parents[0])
                     if len(new_parents) > 1:
-                        msg = dset.repo.format_commit("%B", res_hexsha)
-                        dset.repo.call_git(
+                        msg = ds_repo.format_commit("%B", res_hexsha)
+                        ds_repo.call_git(
                             ["merge", "-m", msg,
                              "--no-ff", "--allow-unrelated-histories"] +
                             new_parents[1:])
-                    head = dset.repo.get_hexsha()
+                    head = ds_repo.get_hexsha()
                     new_bases[res_hexsha] = head
             yield res
             continue
@@ -422,11 +455,11 @@ def _rerun(dset, results, explicit=False):
 
         if new_base:
             if new_base != head:
-                dset.repo.checkout(new_base)
+                ds_repo.checkout(new_base)
                 head_to_restore, head = head, new_base
-        elif parent != head and dset.repo.is_ancestor(onto, parent):
+        elif parent != head and ds_repo.is_ancestor(onto, parent):
             if rerun_action == "run":
-                dset.repo.checkout(parent)
+                ds_repo.checkout(parent)
                 head = parent
             else:
                 _mark_nonrun_result(res, "skip")
@@ -439,15 +472,15 @@ def _rerun(dset, results, explicit=False):
         # We've adjusted base. Now skip, pick, or run the commit.
 
         if rerun_action == "skip-or-pick":
-            if dset.repo.is_ancestor(res_hexsha, head):
+            if ds_repo.is_ancestor(res_hexsha, head):
                 _mark_nonrun_result(res, "skip")
                 if head_to_restore:
-                    dset.repo.checkout(head_to_restore)
+                    ds_repo.checkout(head_to_restore)
                     head, head_to_restore = head_to_restore, None
                 yield res
                 continue
             else:
-                dset.repo.cherry_pick(res_hexsha)
+                ds_repo.cherry_pick(res_hexsha)
                 _mark_nonrun_result(res, "pick")
                 yield res
         elif rerun_action == "run":
@@ -475,12 +508,13 @@ def _rerun(dset, results, explicit=False):
                                  inputs=run_info.get("inputs", []),
                                  extra_inputs=run_info.get("extra_inputs", []),
                                  outputs=outputs,
+                                 assume_ready=assume_ready,
                                  explicit=explicit,
                                  rerun_outputs=auto_outputs,
                                  message=message,
                                  rerun_info=run_info):
                 yield r
-        new_head = dset.repo.get_hexsha()
+        new_head = ds_repo.get_hexsha()
         if new_head not in [head, res_hexsha]:
             new_bases[res_hexsha] = new_head
         head = new_head
@@ -488,9 +522,9 @@ def _rerun(dset, results, explicit=False):
     if branch_to_restore:
         # The user asked us to replay the sequence onto a branch, but the
         # history had merges, so we're in a detached state.
-        dset.repo.update_ref("refs/heads/" + branch_to_restore,
-                             "HEAD")
-        dset.repo.checkout(branch_to_restore)
+        ds_repo.update_ref("refs/heads/" + branch_to_restore,
+                           "HEAD")
+        ds_repo.checkout(branch_to_restore)
 
 
 def _get_rerun_log_msg(res):
@@ -514,13 +548,14 @@ def _get_rerun_log_msg(res):
 
 
 def _report(dset, results):
+    ds_repo = dset.repo
     for res in results:
         if "run_info" in res:
             if res["status"] != "impossible":
                 res["diff"] = list(res["diff"])
                 # Add extra information that is useful in the report but not
                 # needed for the rerun.
-                out = dset.repo.format_commit("%an%x00%aI", res["commit"])
+                out = ds_repo.format_commit("%an%x00%aI", res["commit"])
                 res["author"], res["date"] = out.split("\0")
         yield res
 
@@ -529,6 +564,7 @@ def _get_script_handler(script, since, revision):
     ofh = sys.stdout if script.strip() == "-" else open(script, "w")
 
     def fn(dset, results):
+        ds_repo = dset.repo
         header = """\
 #!/bin/sh
 #
@@ -540,7 +576,7 @@ def _get_script_handler(script, since, revision):
         ofh.write(header.format(
             script=script,
             since="" if since is None else " --since=" + since,
-            revision=dset.repo.get_hexsha(revision),
+            revision=ds_repo.get_hexsha(revision),
             ds='dataset {} at '.format(dset.id) if dset.id else '',
             path=dset.path))
 
@@ -569,7 +605,7 @@ def _get_script_handler(script, since, revision):
                 "\n" + "".join("# " + ln
                                for ln in msg.splitlines(True)) +
                 "\n")
-            commit_descr = dset.repo.describe(res["commit"])
+            commit_descr = ds_repo.describe(res["commit"])
             ofh.write('# (record: {})\n'.format(
                 commit_descr if commit_descr else res["commit"]))
 
